@@ -87,6 +87,8 @@ def set_remote_auth(
         settings.write()
     subprocess.check_call(
         [
+            sys.executable,
+            "-m",
             "dvc",
             "remote",
             "modify",
@@ -99,6 +101,8 @@ def set_remote_auth(
     )
     subprocess.check_call(
         [
+            sys.executable,
+            "-m",
             "dvc",
             "remote",
             "modify",
@@ -166,8 +170,12 @@ def get_remotes(wdir: str = None) -> dict[str, str]:
         return {}
     resp = {}
     for line in out.split("\n"):
-        name, url = line.split("\t")
-        resp[name] = url
+        split_line = line.split()
+        if len(split_line) >= 2:
+            # This is a remote
+            name = split_line[0]
+            url = split_line[1]
+            resp[name] = url
     return resp
 
 
